# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: Compression module for managing context compression.
"""
from agentica.compression.manager import CompressionManager

__all__ = ["CompressionManager"]
